﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using HeuristicLab.Algorithms.DataAnalysis;
using HeuristicLab.Core;
using HeuristicLab.Problems.DataAnalysis;
using HeuristicLab.Problems.Instances.DataAnalysis;
using HeuristicLab.Random;

namespace GaussianProcessDemo {
  public partial class Form1 : Form {
    private IRandom random;
    private ICovarianceFunction covFunction;
    private List<List<double>> data;
    private double[] alpha;


    public Form1() {
      InitializeComponent();
      this.random = new MersenneTwister();

      var sum = new CovarianceSum();
      sum.Terms.Add(new CovarianceSquaredExponentialIso());
      sum.Terms.Add(new CovariancePeriodic());
      sum.Terms.Add(new CovarianceNoise());
      this.covFunction = sum;
      UpdateSliders();

      InitData();
      UpdateChart();
    }

    private void UpdateSliders() {
      flowLayoutPanel1.Controls.Clear();
      for (int i = 0; i < covFunction.GetNumberOfParameters(1); i++) {
        var sliderControl = new TrackBar();
        sliderControl.Minimum = -50;
        sliderControl.Maximum = 50;
        sliderControl.Value = 0;
        sliderControl.ValueChanged += (sender, args) => UpdateChart();
        flowLayoutPanel1.Controls.Add(sliderControl);
      }
    }

    private void InitData() {
      int n = 100;
      data = new List<List<double>>();
      data.Add(ValueGenerator.GenerateSteps(0, 1, 1.0 / n).ToList());

      // sample from GP
      var normalRand = new NormalDistributedRandom(random, 0, 1);
      alpha = (from i in Enumerable.Range(0, n + 1)
               select normalRand.NextDouble()).ToArray();
    }

    private void UpdateChart() {
      var hyp = GetSliderValues();
      var cov = covFunction.GetParameterizedCovarianceFunction(hyp, null);
      var y = Util.SampleGaussianProcess(random, cov, data, alpha);

      chart1.Series[0].Points.Clear();
      foreach (var p in y.Zip(data[0], (t, x) => new { t, x })) {
        chart1.Series[0].Points.AddXY(p.x, p.t);
      }

      var allData = new List<List<double>>();
      allData.Add(y);
      allData.Add(data[0]);
      var variableNames = new string[] { "y", "x" };
      var ds = new Dataset(variableNames, allData);
      var rows = Enumerable.Range(0, data[0].Count);
      var correctModel = new GaussianProcessModel(ds, variableNames.First(), variableNames.Skip(1), rows, hyp, new MeanZero(),
                                                (ICovarianceFunction)covFunction.Clone());
      var yPred = correctModel.GetEstimatedValues(ds, rows);
      chart1.Series[1].Points.Clear();
      foreach (var p in yPred.Zip(data[0], (t, x) => new { t, x })) {
        chart1.Series[1].Points.AddXY(p.x, p.t);
      }
    }

    private double[] GetSliderValues() {
      var hyp = new List<double>();
      foreach (var slider in flowLayoutPanel1.Controls.Cast<TrackBar>()) {
        Console.Write(slider.Value / 10.0 + " ");
        hyp.Add(slider.Value / 10.0);
      }
      Console.WriteLine();

      return hyp.ToArray();
    }
  }
}
